"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WechatWorkAuthMiddleware = void 0;
const common_1 = require("@nestjs/common");
const jsonwebtoken_1 = require("jsonwebtoken");
const queryString = require("query-string");
const services_1 = require("../services");
const constants_1 = require("../constants");
const interfaces_1 = require("../interfaces");
const utils_1 = require("../utils");
let WechatWorkAuthMiddleware = class WechatWorkAuthMiddleware {
    constructor(config, wechatWorkBaseService, wechatWorkContactsService) {
        this.config = config;
        this.wechatWorkBaseService = wechatWorkBaseService;
        this.wechatWorkContactsService = wechatWorkContactsService;
    }
    use(req, res, next) {
        return __awaiter(this, void 0, void 0, function* () {
            const { corpId, agentId } = this.config.baseConfig;
            const { type = interfaces_1.AuthType.COOKIE, returnDomainName, loginPath, logoutPath, loginSuccessPath, loginFailPath, tokenName = constants_1.DEFAULT_TOKEN_NAME, tokenExpires = constants_1.DEFAULT_TOKEN_EXPIRES, jwtSecret, } = this.config.authConfig;
            const loginFailPathObj = queryString.parseUrl(loginFailPath);
            if (req.baseUrl === loginPath) {
                if (req.query.code) {
                    let userIdData;
                    try {
                        userIdData = yield this.wechatWorkBaseService.getUserId(req.query.code);
                    }
                    catch (err) {
                        userIdData = {};
                    }
                    if (!userIdData.UserId) {
                        loginFailPathObj.query.result = interfaces_1.AuthFailResult.QueryUserIdFail;
                        return res.redirect(queryString.stringifyUrl(loginFailPathObj));
                    }
                    let userInfoData;
                    try {
                        userInfoData = yield this.wechatWorkContactsService.getUserInfo(userIdData.UserId);
                    }
                    catch (err) {
                        userInfoData = {};
                    }
                    if (!userInfoData.userid) {
                        loginFailPathObj.query.result = interfaces_1.AuthFailResult.QueryUserInfoFail;
                        return res.redirect(queryString.stringifyUrl(loginFailPathObj));
                    }
                    if (!userInfoData.enable || userInfoData.status !== 1) {
                        loginFailPathObj.query.result = interfaces_1.AuthFailResult.UserInactive;
                        return res.redirect(queryString.stringifyUrl(loginFailPathObj));
                    }
                    const departmentDetail = [];
                    let departmentInfoData;
                    try {
                        departmentInfoData = yield this.wechatWorkContactsService.getAllDepartmentList();
                    }
                    catch (err) {
                        departmentInfoData = {};
                    }
                    if (!departmentInfoData.errcode) {
                        for (const item of userInfoData.department) {
                            departmentDetail.push(utils_1.flatten(utils_1.getPathById(item, departmentInfoData.department)));
                        }
                    }
                    const userData = {
                        userId: userIdData.UserId,
                        name: userInfoData.name,
                        email: userInfoData.email,
                        avatar: userInfoData.avatar,
                        thumb_avatar: userInfoData.thumb_avatar,
                        departmentDetail,
                    };
                    const jwtToken = jsonwebtoken_1.sign(userData, jwtSecret, {
                        expiresIn: tokenExpires,
                    });
                    if (type === interfaces_1.AuthType.COOKIE) {
                        return res.cookie(tokenName, jwtToken, {
                            httpOnly: true,
                            secure: false,
                            expires: new Date(Date.now() + tokenExpires * 1000),
                        }).redirect(loginSuccessPath);
                    }
                    else if (type === interfaces_1.AuthType.CALLBACK_TOKEN) {
                        const parsedPath = queryString.parseUrl(loginSuccessPath, { parseFragmentIdentifier: true });
                        return res.redirect(queryString.stringifyUrl({
                            url: parsedPath.url,
                            query: Object.assign(Object.assign({}, parsedPath.query), { [tokenName]: jwtToken }),
                            fragmentIdentifier: parsedPath.fragmentIdentifier,
                        }));
                    }
                }
                else {
                    if (req.query.state) {
                        loginFailPathObj.query.result = interfaces_1.AuthFailResult.UserRejectQrCode;
                        return res.redirect(queryString.stringifyUrl(loginFailPathObj));
                    }
                    else {
                        return res.redirect(`https://open.work.weixin.qq.com/wwopen/sso/qrConnect?appid=${corpId}&agentid=${agentId}&redirect_uri=${encodeURIComponent(returnDomainName + loginPath)}&state=STATE`);
                    }
                }
            }
            else if (req.baseUrl === logoutPath) {
                if (type === interfaces_1.AuthType.COOKIE) {
                    return res.clearCookie(tokenName).redirect(loginSuccessPath);
                }
                return res.redirect(loginSuccessPath);
            }
            else {
                next();
            }
        });
    }
};
WechatWorkAuthMiddleware = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject(constants_1.WECHAT_WORK_MODULE_CONFIG)),
    __metadata("design:paramtypes", [interfaces_1.WechatWorkConfig,
        services_1.WechatWorkBaseService,
        services_1.WechatWorkContactsService])
], WechatWorkAuthMiddleware);
exports.WechatWorkAuthMiddleware = WechatWorkAuthMiddleware;
