"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var WechatWorkAuthService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.WechatWorkAuthService = void 0;
const common_1 = require("@nestjs/common");
const cookie = require("cookie");
const jsonwebtoken_1 = require("jsonwebtoken");
const interfaces_1 = require("../interfaces");
const constants_1 = require("../constants");
let WechatWorkAuthService = WechatWorkAuthService_1 = class WechatWorkAuthService {
    constructor(config) {
        this.config = config;
        this.logger = new common_1.Logger(WechatWorkAuthService_1.name);
    }
    validateContext(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            let isNoRedirectPath = false;
            for (const item of this.config.authConfig.noRedirectPaths) {
                if (ctx.req.route.path.indexOf(item) === 0) {
                    isNoRedirectPath = true;
                    break;
                }
            }
            let token = '';
            const tokenName = this.config.authConfig.tokenName || constants_1.DEFAULT_TOKEN_NAME;
            const cookies = cookie.parse(ctx.req.headers.cookie || '');
            const tokenFromCookie = cookies[tokenName] || '';
            if (tokenFromCookie) {
                token = tokenFromCookie;
            }
            else {
                const authorizationStr = ctx.req.headers && ctx.req.headers.authorization;
                if (!authorizationStr) {
                    if (isNoRedirectPath) {
                        throw new common_1.HttpException('Not found token', common_1.HttpStatus.UNAUTHORIZED);
                    }
                    else {
                        this.redirectWechatWorkQrCodePage(ctx);
                    }
                }
                const [bearer, tokenFromAuthorization] = authorizationStr.split(' ');
                if (bearer !== 'Bearer' || !tokenFromAuthorization) {
                    if (isNoRedirectPath) {
                        throw new common_1.HttpException('Not found token', common_1.HttpStatus.UNAUTHORIZED);
                    }
                    else {
                        this.redirectWechatWorkQrCodePage(ctx);
                    }
                }
                token = tokenFromAuthorization;
            }
            if (!token) {
                if (isNoRedirectPath) {
                    throw new common_1.HttpException('Not found token', common_1.HttpStatus.UNAUTHORIZED);
                }
                else {
                    this.redirectWechatWorkQrCodePage(ctx);
                }
            }
            ctx.req.user = yield this.validateUserToken(token, ctx, isNoRedirectPath);
            return true;
        });
    }
    validateUserToken(token, ctx, isNoRedirectPath) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const verifiedToken = jsonwebtoken_1.verify(token, this.config.authConfig.jwtSecret);
                if (!verifiedToken || !verifiedToken.userId) {
                    if (isNoRedirectPath) {
                        throw new common_1.HttpException('Invalid token', common_1.HttpStatus.UNAUTHORIZED);
                    }
                    else {
                        this.redirectWechatWorkQrCodePage(ctx);
                    }
                }
                return verifiedToken;
            }
            catch (e) {
                if (isNoRedirectPath) {
                    throw new common_1.HttpException('Invalid token', common_1.HttpStatus.UNAUTHORIZED);
                }
                else {
                    this.redirectWechatWorkQrCodePage(ctx);
                }
            }
        });
    }
    redirectWechatWorkQrCodePage(ctx) {
        const { corpId, agentId } = this.config.baseConfig;
        const { returnDomainName, loginPath } = this.config.authConfig;
        ctx.redirect(`https://open.work.weixin.qq.com/wwopen/sso/qrConnect?appid=${corpId}&agentid=${agentId}&redirect_uri=${encodeURIComponent(returnDomainName + loginPath)}&state=STATE`);
    }
};
WechatWorkAuthService = WechatWorkAuthService_1 = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject(constants_1.WECHAT_WORK_MODULE_CONFIG)),
    __metadata("design:paramtypes", [interfaces_1.WechatWorkConfig])
], WechatWorkAuthService);
exports.WechatWorkAuthService = WechatWorkAuthService;
